#include <stdio.h>
#include "Arvore23.h"
#define NDEBUG
#include <assert.h>

extern void Arvore23LiberaNo (NODE_23_T *tree);

NODE_23_T *Arvore23RemoveRec (
	NODE_23_T *tree
	, const void *valor
	, int (*cmp)(const void *, const void *)
	, const void **removeu
	, int pos_filho
	, NODE_23_T *pai /* n pai a descer valor caso o n filho atual fique sem elementos */
	, int pos_v_pai
	, NODE_23_T *irmao /* n irmo a fundir caso o n filho atual fique sem elementos */
	, int pos_irmao
)
{
	int comp;
	int pos;
	NODE_23_T *no;
	
	assert (tree != NULL && valor != NULL && cmp != NULL && removeu != NULL);
	
	/* Procurar posio do elemento */
	if ((comp = cmp (valor, tree->v[0])) < 0) pos = 0;
	else if (comp == 0) pos = -1;
	else if (tree->v[1] == NULL || (comp = cmp (valor, tree->v[1])) < 0) pos = 1;
	else if (comp == 0) pos = -2;
	else pos = 2;

	if (pos < 0) { /* achou elemento */
		pos = -1 - pos;
		if (*removeu == NULL) *removeu = tree->v[pos]; /* se ja tinha conteudo,  porque removeu em ns no folha anteriormente */
		if (tree->f[0] == NULL) {
			/*  no n folha */
			if (pos == 0) tree->v[0] = tree->v[1];
			tree->v[1] = NULL;
		}
		else {
			/* no  n folha. Buscar valor imediatamente superior para substituir, e depois exclu-lo. */
			no = tree->f[pos + 1];
			while (no->f[0] != NULL) no = no->f[0];
			tree->v[pos] = no->v[0];
			/* irmao  esquerda */
			tree->f[pos + 1] = Arvore23RemoveRec (tree->f[pos + 1], tree->v[pos], cmp, removeu, pos + 1, tree, pos, tree->f[pos], pos);
		}
	}
	else { /* Nao achou o elemento */
		if (tree->f[0] == NULL) return tree; /*  n folha */
		/* mandar irmao  direita se a subrvore for a primeira */
		if (pos == 0) tree->f[pos] = Arvore23RemoveRec (tree->f[pos], valor, cmp, removeu, pos, tree, pos, tree->f[pos + 1], pos + 1);
		else tree->f[pos] = Arvore23RemoveRec (tree->f[pos], valor, cmp, removeu, pos, tree, pos - 1, tree->f[pos - 1], pos - 1);
	}
	
	if (tree->v[0] != NULL) return tree;
	
	/* a partir daqui o n ficou vazio, e deve-se fazer os ajustes necessarios */
	if (pai == NULL) { /* no raiz ficou vazio, diminuir rvore. */
		no = tree->f[0];
		Arvore23LiberaNo (tree);
		return no;
	}
	if (irmao->v[1] != NULL) {
		/* apenas remanejar */
		tree->v[0] = pai->v[pos_v_pai];
		if (pos_filho < pos_irmao) {
			tree->f[1] = irmao->f[0];
			pai->v[pos_v_pai] = irmao->v[0];
			irmao->v[0] = irmao->v[1];
			irmao->f[0] = irmao->f[1];
			irmao->f[1] = irmao->f[2];
		}
		else {
			tree->f[1] = tree->f[0];
			tree->f[0] = irmao->f[2];
			pai->v[pos_v_pai] = irmao->v[1];
		}
		irmao->f[2] = NULL;
		irmao->v[1] = NULL;
	}
	else {
		/* fundir n filho atual com n irmo, pegando um elemento do pai */
		if (pos_filho < pos_irmao) {
			tree->v[0] = pai->v[pos_v_pai];
			tree->f[1] = irmao->f[0];
			tree->v[1] = irmao->v[0];
			tree->f[2] = irmao->f[1];
			Arvore23LiberaNo (irmao);
			if (pos_v_pai == 0) {
				pai->v[0] = pai->v[1];
				pai->f[1] = pai->f[2];
			}
		}
		else {
			tree->f[2] = tree->f[0];
			tree->v[1] = pai->v[pos_v_pai];
			tree->f[1] = irmao->f[1];
			tree->v[0] = irmao->v[0];
			tree->f[0] = irmao->f[0];
			Arvore23LiberaNo (irmao);
			if (pos_v_pai == 0) {
				pai->f[0] = pai->f[1];
				pai->v[0] = pai->v[1];
			}
			pai->f[1] = pai->f[2];
		}
		pai->f[2] = NULL;
		pai->v[1] = NULL;
	}
	return pai->f[pos_filho];
}

/* devolve o elemento removido, ou NULL se no foi encontrado */
const void *Arvore23Remove (ARVORE_23_T tree23, const void *valor)
{
	const void *removeu;
	if (tree23 == NULL || valor == NULL) return NULL;
	if (tree23->tree == NULL) return NULL;
	
	removeu = NULL;
	tree23->tree = Arvore23RemoveRec (tree23->tree, valor, tree23->cmp, &removeu, 0, NULL, 0, NULL, 0);
	if (removeu != NULL) tree23->n--;
	return removeu;
}
