#include <stdio.h>
#include "Arvore23.h"

int Arvore23ValidaRec (
	NODE_23_T *tree
	, int (*cmp)(const void *, const void *)
	, int folha /* se 0,  folha */
	, const void *v_min
	, const void *v_max
)
{
	if (tree == NULL) return 0; /* subrvores no podem ser vazias, pois folhas esto terminando a recurso */

	if (tree->v[0] == NULL) return 0;
	if (v_min != NULL) {
		if (cmp (tree->v[0], v_min) <= 0) return 0;
		if (tree->v[1] != NULL && cmp (tree->v[1], v_min) <= 0) return 0;
	}
	if (v_max != NULL) {
		if (cmp (tree->v[0], v_max) >= 0) return 0;
		if (tree->v[1] != NULL && cmp (tree->v[1], v_max) >= 0) return 0;
	}
	if (tree->v[1] != NULL && cmp (tree->v[0], tree->v[1]) >= 0) return 0;
	if (folha == 0) {
		if (tree->f[0] != NULL || tree->f[1] != NULL || tree->f[2] != NULL) return 0;
		return 1;
	}
	/* a partir daqui no  n folha */
	if (tree->f[0] == NULL || tree->f[1] == NULL) return 0;
	if (tree->f[2] == NULL && tree->v[1] != NULL) return 0;
	if (tree->f[2] != NULL && tree->v[1] == NULL) return 0;
	if (Arvore23ValidaRec (tree->f[0], cmp, folha - 1, v_min, tree->v[0]) != 1) return 0;
	if (Arvore23ValidaRec (tree->f[1], cmp, folha - 1, tree->v[0], (tree->v[1] == NULL ? v_max : tree->v[1])) != 1) return 0;
	if (tree->v[1] != NULL && Arvore23ValidaRec (tree->f[2], cmp, folha - 1, tree->v[1], v_max) != 1) return 0;
	return 1;
}

/* Devolve 1 se for uma rvore vlida, ou 0 caso contrrio.
Esta funo pode ser usada quando houver desconfiana de que a estrutura da rvore
est com problema (isso pode acontecer caso as chaves internas dos ponteiros tenham
sido alteradas externamente).
Obs: Tempo de execuo  linear em n (T(n)), onde n  o nmero de ns.
*/
int Arvore23Valida (ARVORE_23_T tree23)
{
	int nivel_folha;
	NODE_23_T *no;
	if (tree23->tree == NULL) return 1; /* rvore vazia  vlida. */
	nivel_folha = 0;
	no = tree23->tree;
	while (no->f[0] != NULL) {
		nivel_folha++;
		no = no->f[0];
	}
	return Arvore23ValidaRec (tree23->tree, tree23->cmp, nivel_folha, NULL, NULL);
}
